@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 # A resuming media player for Podcasts and Audiobooks
 # Copyright (c) 2008-05-26 Thomas Perl <thpinfo.com>
 #
@@ -44,7 +45,7 @@ except:
 about_name = 'Panucci'
 about_text = _('Resuming audiobook and podcast player')
 about_authors = ['Thomas Perl', 'Nick (nikosapi)', 'Matthew Taylor']
-about_website = 'http://thpinfo.com/2008/panucci/'
+about_website = 'http://panucci.garage.maemo.org/'
 donate_wishlist_url = 'http://www.amazon.de/gp/registry/2PD2MYGHE6857'
 donate_device_url = 'http://maemo.gpodder.org/donate.html'
 
@@ -111,15 +112,19 @@ class PositionManager(object):
             self.positions = {}
 
     def set_position(self, url, position):
+        log('set_position(%s, %d)' % (url, position))
         if not url in self.positions:
             self.positions[url] = {}
 
         self.positions[url]['position'] = position
 
     def get_position(self, url):
+        log('get_position(%s)' % url)
         if url in self.positions and 'position' in self.positions[url]:
+            log('=> %d' % (self.positions[url]['position']))
             return self.positions[url]['position']
         else:
+            log('=> NULL!!!!!')
             return 0
 
     def set_bookmarks(self, url, bookmarks):
@@ -140,8 +145,6 @@ class PositionManager(object):
         pickle.dump(self.positions, f)
         f.close()
 
-pm = PositionManager()
-
 class BookmarksWindow(gtk.Window):
     def __init__(self, main):
         self.main = main
@@ -197,7 +200,7 @@ class BookmarksWindow(gtk.Window):
         self.hbox.pack_start(self.close_button)
         self.vbox.pack_start(self.hbox, False, True)
         self.add(self.vbox)
-        for label, pos in pm.get_bookmarks(self.main.filename):
+        for label, pos in self.main.pm.get_bookmarks(self.main.filename):
             self.add_bookmark(label=label, pos=pos)
         self.show_all()
 
@@ -205,7 +208,7 @@ class BookmarksWindow(gtk.Window):
         bookmarks = []
         for row in self.model:
             bookmarks.append((row[0], row[2]))
-        pm.set_bookmarks(self.main.filename, bookmarks)
+        self.main.pm.set_bookmarks(self.main.filename, bookmarks)
         self.destroy()
 
     def label_edited(self, cellrenderer, path, new_text):
@@ -229,6 +232,8 @@ class BookmarksWindow(gtk.Window):
         if iter is not None:
             pos = model.get_value(iter, 2)
             self.main.do_seek(from_beginning=pos)
+            if not self.main.playing:
+                self.main.start_stop(None)
 
 class GTK_Main(dbus.service.Object):
 
@@ -236,17 +241,20 @@ class GTK_Main(dbus.service.Object):
         dbus.service.Object.__init__(self, object_path="/player",
             bus_name=bus_name)
 
-        self.gconf_client = gconf.client_get_default()
-        self.gconf_client.add_dir('/apps/panucci', gconf.CLIENT_PRELOAD_NONE)
-        self.gconf_client.notify_add('/apps/panucci', self.gconf_key_changed)
-
         self.filename = filename
         self.progress_timer_id = None
         self.volume_timer_id = None
+
+        self.gconf_client = gconf.client_get_default()
         self.make_main_window()
+
+        self.gconf_client.add_dir('/apps/panucci', gconf.CLIENT_PRELOAD_NONE)
+        self.gconf_client.notify_add('/apps/panucci', self.gconf_key_changed)
+
         self.has_coverart = False
         self.has_id3_coverart = False
         self.playing = False
+        self.player_last_position = (0,0)
 
         if running_on_tablet:
             # Enable play/pause with headset button
@@ -263,7 +271,7 @@ class GTK_Main(dbus.service.Object):
         self.get_volume_level = lambda: 0
         self.set_volume_level = lambda x: 0
 
-        vol = pm.get_position('volume')
+        vol = self.pm.get_position('volume')
         if vol < 0 or vol > 1 : vol = 0.2
         self.set_volume(vol)
 
@@ -282,14 +290,22 @@ class GTK_Main(dbus.service.Object):
             window = gtk.Window(gtk.WINDOW_TOPLEVEL)
 
         window.set_title('Panucci')
-        self.window_icon = find_image('panucci.png')
-        if self.window_icon is not None:
-            window.set_icon_from_file( self.window_icon )
         window.set_default_size(400, -1)
         window.set_border_width(0)
         window.connect("destroy", self.destroy)
+
+        window.show()
+        while gtk.events_pending():
+            gtk.main_iteration(False)
+
+        self.window_icon = find_image('panucci.png')
+        if self.window_icon is not None:
+            window.set_icon_from_file( self.window_icon )
+
         self.main_window = window
 
+        self.pm = PositionManager()
+
         if running_on_tablet:
             window.set_menu(self.create_menu())
         else:
@@ -328,13 +344,13 @@ class GTK_Main(dbus.service.Object):
         metadata_vbox.set_spacing(8)
         empty_label = gtk.Label()
         metadata_vbox.pack_start(empty_label, True, True)
-        self.artist_label = gtk.Label('')
+        self.artist_label = gtk.Label('Welcome to Panucci!')
         self.artist_label.set_ellipsize(pango.ELLIPSIZE_END)
         metadata_vbox.pack_start(self.artist_label, False, False)
-        self.album_label = gtk.Label('')
+        self.album_label = gtk.Label('© 2008 thp, nikosapi and matt')
         self.album_label.set_ellipsize(pango.ELLIPSIZE_END)
         metadata_vbox.pack_start(self.album_label, False, False)
-        self.title_label = gtk.Label('')
+        self.title_label = gtk.Label('http://panucci.garage.maemo.org/')
         self.title_label.set_line_wrap(True)
         metadata_vbox.pack_start(self.title_label, False, False)
         empty_label = gtk.Label()
@@ -483,24 +499,27 @@ class GTK_Main(dbus.service.Object):
         dialog.destroy()
 
     def save_position(self):
+        log('in save_positoin')
         try:
             (pos, dur) = self.player_get_position()
-            pm.set_position(self.filename, pos)
-        except:
-            pass
+            self.pm.set_position(self.filename, pos)
+        except Exception, e:
+            log(e)
 
     def get_position(self, pos=None):
         if pos is None:
             try:
                 (pos, dur) = self.player_get_position()
-            except:
-                pos = 0
+            except Exception, e:
+                log(e)
+                log('get_position exception')
+                pos = 99000000000
         text = self.convert_ns(pos)
         return (text, pos)
 
     def destroy(self, widget):
         self.stop_playing()
-        pm.set_position( 'volume', self.get_volume())
+        self.pm.set_position( 'volume', self.get_volume())
         gtk.main_quit()
 
     def gconf_key_changed(self, client, connection_id, entry, args):
@@ -610,7 +629,6 @@ class GTK_Main(dbus.service.Object):
         self.setup_player(self.filename)
 
         self.has_coverart = False
-        self.want_to_seek = True
         self.start_playback()
         self.start_progress_timer()
 
@@ -621,7 +639,9 @@ class GTK_Main(dbus.service.Object):
     def open_file_callback(self, widget=None):
         old_filename = self.filename
         filename = self.get_file_from_filechooser()
+        self.player_last_position = (0,0)
         if filename is not None:
+            self.want_to_seek = True
             self.play_file(filename)
 
     @dbus.service.method('org.panucci.interface')
@@ -641,7 +661,7 @@ class GTK_Main(dbus.service.Object):
         image(self.button, gtk.STOCK_OPEN, True)
 
     def start_playback(self):
-        self.set_controls_sensitivity(True)
+        self.set_controls_sensitivity(False)
         for widget in [ self.title_label, self.artist_label, self.album_label ]:
             widget.set_text('')
             widget.hide()
@@ -652,6 +672,7 @@ class GTK_Main(dbus.service.Object):
         if filename.lower().endswith('.ogg') and running_on_tablet:
             log( 'Using OGG workaround, I hope this works...' )
 
+            self.want_to_seek = True
             self.player = gst.Pipeline('player')
             source = gst.element_factory_make('gnomevfssrc', 'file-source')
             audio_decoder = gst.element_factory_make('tremor', 'vorbis-decoder')
@@ -668,6 +689,7 @@ class GTK_Main(dbus.service.Object):
         else:
             log( 'Using plain-old playbin.' )
 
+            self.want_to_seek = True
             self.player = gst.element_factory_make('playbin', 'player')
 
             # Workaround for volume on maemo, they use a 0 to 10 scale
@@ -714,7 +736,16 @@ class GTK_Main(dbus.service.Object):
         if self.filename is None or not os.path.exists(self.filename):
             self.open_file_callback()
             return
-
+        
+        if self.playing:
+            try:
+                self.player_last_position = (self.player.query_position(self.time_format, None)[0],
+                                             self.player.query_duration(self.time_format, None)[0])
+            except Exception, e:
+                log('in start_stop')
+                log(e)
+                log('in start_stop')
+            self.set_controls_sensitivity(False)
         self.playing = not self.playing
 
         if self.playing:
@@ -733,30 +764,40 @@ class GTK_Main(dbus.service.Object):
                 from_current=n: seek n nanoseconds from the current position
                 percent=n: seek n percent from the beginning of the file
         """
+        log('do_seek(from_beginning=%s, from_current=%s, percent=%s)' % (repr(from_beginning), repr(from_current), repr(percent)))
         self.want_to_seek = True
+        self.set_controls_sensitivity(False)
 
-        position, duration = self.player_get_position()
-        # if position and duration are 0 then player_get_position caught an
-        # exception. Therefore self.player isn't ready to be seeing.
-        if not ( position or duration ) or self.player is None:
+        if self.player is None:
+            log('player is none at beginning of do_seek')
             return False
 
+        position, duration = self.player_get_position()
+        log('position and duration in do_seek: %d, %d' % (position, duration))
+
         if from_beginning is not None:
-            assert from_beginning >= 0
-            position = min( from_beginning, duration )
+            position = from_beginning
         elif from_current is not None:
-            position = max( 0, min( position+from_current, duration ))
+            position += from_current
+            if position < 0:
+                position = 0
         elif percent is not None:
-            assert 0 <= percent <= 1
             position = int(duration*percent)
         else:
             log('No seek parameters specified.')
             return False
 
         # Preemptively update the progressbar to make seeking nice and smooth
-        self.set_progress_callback( position, duration )
-        self.player.seek_simple(self.time_format, gst.SEEK_FLAG_FLUSH, position)
-        self.want_to_seek = False
+        self.set_progress_callback(position, duration)
+        def cb(position, duration):
+            log('callback for setting the position called')
+            log('position = %d' % position)
+            self.player.seek_simple(self.time_format, gst.SEEK_FLAG_FLUSH, position)
+            self.player_last_position = (position, duration)
+            self.want_to_seek = False
+            return False
+        log('position before cb calling: %d' % position)
+        gobject.idle_add(cb, position, duration)
 
         return True
 
@@ -765,8 +806,14 @@ class GTK_Main(dbus.service.Object):
         try:
             pos_int = self.player.query_position(self.time_format, None)[0]
             dur_int = self.player.query_duration(self.time_format, None)[0]
-        except:
+        except Exception, e:
+            log(e)
             pos_int = dur_int = 0
+
+        if pos_int < 1000000000 and (dur_int == self.player_last_position[1] or dur_int == 0):
+            log('fixing pos_int')
+            pos_int = self.player_last_position[0]
+
         return pos_int, dur_int
 
     def progress_timer_callback( self ):
@@ -775,6 +822,8 @@ class GTK_Main(dbus.service.Object):
             # This prevents bogus values from being set while seeking
             if ( pos_int > 10**9 ) and ( dur_int > 10**9 ):
                 self.set_progress_callback( pos_int, dur_int )
+        else:
+            log('self.playing = %s, self.want_to_seek = %s' % (str(self.playing), str(self.want_to_seek)))
         return True
 
     def start_progress_timer( self ):
@@ -793,7 +842,7 @@ class GTK_Main(dbus.service.Object):
 
         if t == gst.MESSAGE_EOS:
             self.stop_playing()
-            pm.set_position(self.filename, 0)
+            self.pm.set_position(self.filename, 0)
 
         elif t == gst.MESSAGE_ERROR:
             err, debug = message.parse_error()
@@ -806,7 +855,8 @@ class GTK_Main(dbus.service.Object):
 
                 if self.want_to_seek:
                     # This only gets called when the file is first loaded
-                    self.do_seek(from_beginning=pm.get_position(self.filename))
+                    log('i want to seek!')
+                    self.do_seek(from_beginning=self.pm.get_position(self.filename))
                 else:
                     self.set_controls_sensitivity(True)
 
@@ -915,10 +965,10 @@ class GTK_Main(dbus.service.Object):
 def run(filename=None):
     session_bus = dbus.SessionBus(mainloop=dbus.glib.DBusGMainLoop())
     bus_name = dbus.service.BusName('org.panucci', bus=session_bus)
-    GTK_Main(bus_name, filename)
+    m = GTK_Main(bus_name, filename)
     gtk.main()
     # save position manager data
-    pm.save()
+    m.pm.save()
 
 if __name__ == '__main__':
     log( 'WARNING: Use the "panucci" executable, to run this program.' )
